@@ -1,18 +1,18 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-
-from trytond.model.fields.field import Field
-from trytond.pyson import PYSON
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from trytond.pyson import PYSONEncoder
+from .field import Field
+from ...pyson import PYSON
 
 
 def digits_validate(value):
     if value:
         assert isinstance(value, tuple), 'digits must be a tuple'
         for i in value:
-            assert isinstance(i, (int, long, PYSON)), \
+            assert isinstance(i, (int, PYSON)), \
                 'digits must be tuple of integers or PYSON'
             if isinstance(i, PYSON):
-                assert i.types().issubset(set([int, long])), \
+                assert i.types().issubset(set([int, int])), \
                     'PYSON digits must return an integer'
 
 
@@ -21,11 +21,12 @@ class Float(Field):
     Define a float field (``float``).
     '''
     _type = 'float'
+    _sql_type = 'FLOAT'
 
     def __init__(self, string='', digits=None, help='', required=False,
             readonly=False, domain=None, states=None, select=False,
             on_change=None, on_change_with=None, depends=None,
-            order_field=None, context=None, loading='eager'):
+            context=None, loading='eager'):
         '''
         :param digits: a list of two integers defining the total
             of digits and the number of decimals of the float.
@@ -33,8 +34,7 @@ class Float(Field):
         super(Float, self).__init__(string=string, help=help,
             required=required, readonly=readonly, domain=domain, states=states,
             select=select, on_change=on_change, on_change_with=on_change_with,
-            depends=depends, order_field=order_field, context=context,
-            loading=loading)
+            depends=depends, context=context, loading=loading)
         self.__digits = None
         self.digits = digits
 
@@ -48,3 +48,14 @@ class Float(Field):
         self.__digits = value
 
     digits = property(_get_digits, _set_digits)
+
+    def sql_format(self, value):
+        if value is None:
+            return None
+        return float(value)
+
+    def definition(self, model, language):
+        encoder = PYSONEncoder()
+        definition = super().definition(model, language)
+        definition['digits'] = encoder.encode(self.digits)
+        return definition
