@@ -9,6 +9,12 @@ class ModelSingleton(ModelStorage):
     Define a singleton model in Tryton.
     """
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        # Cache disable because it is used as a read by the client
+        cls.__rpc__['default_get'].cache = None
+
     @classmethod
     def get_singleton(cls):
         '''
