@@ -1,31 +1,31 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from ..pool import Pool
+
 from .group import *
 from .user import *
-from .request import *
 from .ir import *
+from . import routes
+
+__all__ = ['register', 'routes']
 
 
 def register():
     Pool.register(
         Group,
         User,
-        Group2,
+        LoginAttempt,
         UserAction,
         UserGroup,
         Warning_,
+        UserApplication,
         UserConfigStart,
-        Request,
-        RequestLink,
-        RequestHistory,
-        RequestReference,
         UIMenuGroup,
         ActionGroup,
-        ModelFieldGroup,
         ModelButtonGroup,
+        ModelButtonRule,
+        ModelButtonClick,
         RuleGroupGroup,
-        RuleGroupUser,
         Lang,
         SequenceType,
         SequenceTypeGroup,
@@ -36,3 +36,6 @@ def register():
     Pool.register(
         UserConfig,
         module="res", type_='wizard')
+    Pool.register(
+        EmailResetPassword,
+        module='res', type_='report')
