@@ -4,12 +4,17 @@
 import collections
 import json
 import logging
-import os
 import select
 import threading
 import time
 import uuid
+try:
+    from http import HTTPStatus
+except ImportError:
+    from http import client as HTTPStatus
+from urllib.parse import urljoin
 
+from werkzeug.utils import redirect
 from werkzeug.wrappers import Response
 from werkzeug.exceptions import NotImplemented, BadRequest
 
@@ -28,6 +33,8 @@ _cache_timeout = config.getint('bus', 'cache_timeout')
 _select_timeout = config.getint('bus', 'select_timeout')
 _long_polling_timeout = config.getint('bus', 'long_polling_timeout')
 _allow_subscribe = config.getboolean('bus', 'allow_subscribe')
+_url_host = config.get('bus', 'url_host')
+_web_cache_timeout = config.getint('web', 'cache_timeout')
 
 
 class _MessageQueue:
@@ -36,7 +43,7 @@ class _MessageQueue:
 
     def __init__(self, timeout):
         super().__init__()
-        self._lock = collections.defaultdict(threading.Lock)
+        self._lock = threading.Lock()
         self._timeout = timeout
         self._messages = []
 
@@ -66,7 +73,7 @@ class _MessageQueue:
             if first_message and not found:
                 message = first_message
 
-        with self._lock[os.getpid()]:
+        with self._lock:
             del self._messages[:to_delete_index]
 
         return message.channel, message.content
@@ -75,21 +82,20 @@ class _MessageQueue:
 class LongPollingBus:
 
     _channel = 'bus'
-    _queues_lock = collections.defaultdict(threading.Lock)
+    _queues_lock = threading.Lock()
     _queues = collections.defaultdict(
         lambda: {'timeout': None, 'events': collections.defaultdict(list)})
     _messages = {}
 
     @classmethod
     def subscribe(cls, database, channels, last_message=None):
-        pid = os.getpid()
-        with cls._queues_lock[pid]:
-            start_listener = (pid, database) not in cls._queues
-            cls._queues[pid, database]['timeout'] = time.time() + _db_timeout
+        with cls._queues_lock:
+            start_listener = database not in cls._queues
+            cls._queues[database]['timeout'] = time.time() + _db_timeout
             if start_listener:
                 listener = threading.Thread(
                     target=cls._listen, args=(database,), daemon=True)
-                cls._queues[pid, database]['listener'] = listener
+                cls._queues[database]['listener'] = listener
                 listener.start()
 
         messages = cls._messages.get(database)
@@ -100,12 +106,11 @@ class LongPollingBus:
 
         event = threading.Event()
         for channel in channels:
-            if channel in cls._queues[pid, database]['events']:
-                event_channel = cls._queues[pid, database]['events'][channel]
+            if channel in cls._queues[database]['events']:
+                event_channel = cls._queues[database]['events'][channel]
             else:
-                with cls._queues_lock[pid]:
-                    event_channel = cls._queues[pid, database][
-                        'events'][channel]
+                with cls._queues_lock:
+                    event_channel = cls._queues[database]['events'][channel]
             event_channel.append(event)
 
         triggered = event.wait(_long_polling_timeout)
@@ -115,9 +120,9 @@ class LongPollingBus:
             response = cls.create_response(
                 *cls._messages[database].get_next(channels, last_message))
 
-        with cls._queues_lock[pid]:
+        with cls._queues_lock:
             for channel in channels:
-                events = cls._queues[pid, database]['events'][channel]
+                events = cls._queues[database]['events'][channel]
                 for e in events[:]:
                     if e.is_set():
                         events.remove(e)
@@ -141,16 +146,16 @@ class LongPollingBus:
             raise NotImplemented
 
         logger.info("listening on channel '%s'", cls._channel)
-        conn = db.get_connection(autocommit=True)
-        pid = os.getpid()
+        conn = db.get_connection()
         try:
             cursor = conn.cursor()
             cursor.execute('LISTEN "%s"' % cls._channel)
+            conn.commit()
 
             cls._messages[database] = messages = _MessageQueue(_cache_timeout)
 
             now = time.time()
-            while cls._queues[pid, database]['timeout'] > now:
+            while cls._queues[database]['timeout'] > now:
                 readable, _, _ = select.select([conn], [], [], _select_timeout)
                 if not readable:
                     continue
@@ -165,10 +170,10 @@ class LongPollingBus:
                     message = payload['message']
                     messages.append(channel, message)
 
-                    with cls._queues_lock[pid]:
-                        events = cls._queues[pid, database][
-                            'events'][channel].copy()
-                        cls._queues[pid, database]['events'][channel].clear()
+                    with cls._queues_lock:
+                        events = \
+                            cls._queues[database]['events'][channel].copy()
+                        cls._queues[database]['events'][channel].clear()
                     for event in events:
                         event.set()
                 now = time.time()
@@ -176,20 +181,20 @@ class LongPollingBus:
             logger.error('bus listener on "%s" crashed', database,
                 exc_info=True)
 
-            with cls._queues_lock[pid]:
-                del cls._queues[pid, database]
+            with cls._queues_lock:
+                del cls._queues[database]
             raise
         finally:
             db.put_connection(conn)
 
-        with cls._queues_lock[pid]:
-            if cls._queues[pid, database]['timeout'] <= now:
-                del cls._queues[pid, database]
+        with cls._queues_lock:
+            if cls._queues[database]['timeout'] <= now:
+                del cls._queues[database]
             else:
                 # A query arrived between the end of the while and here
                 listener = threading.Thread(
                     target=cls._listen, args=(database,), daemon=True)
-                cls._queues[pid, database]['listener'] = listener
+                cls._queues[database]['listener'] = listener
                 listener.start()
 
     @classmethod
@@ -219,6 +224,13 @@ else:
 def subscribe(request, database_name):
     if not _allow_subscribe:
         raise NotImplemented
+    if _url_host and _url_host != request.host_url:
+        response = redirect(
+            urljoin(_url_host, request.path), HTTPStatus.PERMANENT_REDIRECT)
+        # Allow to change the redirection after some time
+        response.headers['Cache-Control'] = (
+            'private, max-age=%s' % _web_cache_timeout)
+        return response
     user = request.authorization.get('userid')
     channels = request.parsed_data.get('channels', [])
     if user is None:
