@@ -5,7 +5,10 @@ from functools import wraps
 
 from sql.operators import NotIn
 
-from trytond.model import ModelView, ModelSQL, fields, Unique
+from trytond.exceptions import UserError
+from trytond.i18n import gettext
+from trytond.model import ModelView, ModelSQL, fields, Unique, sequence_ordered
+from trytond.model.exceptions import AccessError
 from trytond.modules import get_module_list, get_module_info
 from trytond.wizard import Wizard, StateView, Button, StateTransition, \
     StateAction
@@ -24,6 +27,10 @@ __all__ = [
     ]
 
 
+class DeactivateDependencyError(UserError):
+    pass
+
+
 def filter_state(state):
     def filter(func):
         @wraps(func)
@@ -65,12 +72,6 @@ class Module(ModelSQL, ModelView):
         cls.__rpc__.update({
                 'on_write': RPC(instantiate=0),
                 })
-        cls._error_messages.update({
-            'delete_state': ('You can not remove a module that is activated '
-                    'or will be activated'),
-            'deactivate_dep': ('Some activated modules depend on the ones '
-                    'you are trying to deactivate:'),
-            })
         cls._buttons.update({
                 'activate': {
                     'invisible': Eval('state') != 'not activated',
@@ -176,7 +177,7 @@ class Module(ModelSQL, ModelView):
                     'to remove',
                     'to activate',
                     ):
-                cls.raise_user_error('delete_state')
+                raise AccessError(gettext('ir.msg_module_delete_state'))
         return super(Module, cls).delete(records)
 
     @classmethod
@@ -265,9 +266,9 @@ class Module(ModelSQL, ModelView):
                         module_table.state, ['not activated', 'to remove'])))
             res = cursor.fetchall()
             if res:
-                cls.raise_user_error('deactivate_dep',
-                        error_description='\n'.join(
-                            '\t%s: %s' % (x[0], x[1]) for x in res))
+                raise DeactivateDependencyError(
+                    gettext('ir.msg_module_deactivate_dependency'),
+                    '\n'.join('\t%s: %s' % (x[0], x[1]) for x in res))
         cls.write(modules, {'state': 'to remove'})
 
     @classmethod
@@ -381,22 +382,16 @@ class ModuleDependency(ModelSQL, ModelView):
             return 'unknown'
 
 
-class ModuleConfigWizardItem(ModelSQL, ModelView):
+class ModuleConfigWizardItem(sequence_ordered(), ModelSQL, ModelView):
     "Config wizard to run after activating a module"
     __name__ = 'ir.module.config_wizard.item'
     action = fields.Many2One('ir.action', 'Action', required=True,
         readonly=True)
-    sequence = fields.Integer('Sequence', required=True)
     state = fields.Selection([
         ('open', 'Open'),
         ('done', 'Done'),
         ], string='State', required=True, select=True)
 
-    @classmethod
-    def __setup__(cls):
-        super(ModuleConfigWizardItem, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
-
     @classmethod
     def __register__(cls, module_name):
         TableHandler = backend.get('TableHandler')
@@ -417,6 +412,11 @@ class ModuleConfigWizardItem(ModelSQL, ModelView):
 
         super(ModuleConfigWizardItem, cls).__register__(module_name)
 
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: remove required on sequence
+        table.not_null_action('sequence', 'remove')
+
     @staticmethod
     def default_state():
         return 'open'
