@@ -1,5 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+from collections import defaultdict
+
+from trytond.i18n import gettext
+from trytond.model.exceptions import ValidationError
 from ..model import ModelView, ModelSQL, fields, EvalEnvironment, Check
 from ..transaction import Transaction
 from ..cache import Cache
@@ -11,10 +15,16 @@ __all__ = [
     ]
 
 
+class DomainError(ValidationError):
+    pass
+
+
 class RuleGroup(ModelSQL, ModelView):
     "Rule group"
     __name__ = 'ir.rule.group'
-    name = fields.Char('Name', select=True)
+    name = fields.Char(
+        "Name", select=True, translate=True, required=True,
+        help="Displayed to users when access error is raised for this rule.")
     model = fields.Many2One('ir.model', 'Model', select=True,
         required=True, ondelete='CASCADE')
     global_p = fields.Boolean('Global', select=True,
@@ -98,12 +108,7 @@ class Rule(ModelSQL, ModelView):
         '- "user" as the current user')
     _domain_get_cache = Cache('ir_rule.domain_get', context=False)
 
-    @classmethod
-    def __setup__(cls):
-        super(Rule, cls).__setup__()
-        cls._error_messages.update({
-                'invalid_domain': 'Invalid domain in rule "%s".',
-                })
+    modes = {'read', 'write', 'create', 'delete'}
 
     @classmethod
     def validate(cls, rules):
@@ -117,22 +122,23 @@ class Rule(ModelSQL, ModelView):
             try:
                 value = PYSONDecoder(ctx).decode(rule.domain)
             except Exception:
-                cls.raise_user_error('invalid_domain', (rule.rec_name,))
+                raise DomainError(gettext(
+                        'ir.msg_rule_invalid_domain', name=rule.rec_name))
             if not isinstance(value, list):
-                cls.raise_user_error('invalid_domain', (rule.rec_name,))
+                raise DomainError(gettext(
+                        'ir.msg_rule_invalid_domain', name=rule.rec_name))
             else:
                 try:
                     fields.domain_validate(value)
                 except Exception:
-                    cls.raise_user_error('invalid_domain', (rule.rec_name,))
+                    raise DomainError(gettext(
+                            'ir.msg_rule_invalid_domain', name=rule.rec_name))
 
     @staticmethod
     def _get_context():
         User = Pool().get('res.user')
-        transaction = Transaction()
-        user_id = transaction.user
-        with transaction.set_context(_check_access=False, _datetime=None), \
-                transaction.set_user(0):
+        user_id = Transaction().user
+        with Transaction().set_context(_check_access=False, _datetime=None):
             user = EvalEnvironment(User(user_id), User)
         return {
             'user': user,
@@ -144,35 +150,29 @@ class Rule(ModelSQL, ModelView):
         return (Transaction().user, Transaction().context.get('_datetime'))
 
     @classmethod
-    def domain_get(cls, model_name, mode='read'):
-        assert mode in ['read', 'write', 'create', 'delete'], \
-            'Invalid domain mode for security'
-
-        # root user above constraint
-        if Transaction().user == 0:
-            if not Transaction().context.get('user'):
-                return
-            with Transaction().set_user(Transaction().context['user']):
-                return cls.domain_get(model_name, mode=mode)
-
-        key = (model_name, mode) + cls._get_cache_key()
-        domain = cls._domain_get_cache.get(key, False)
-        if domain is not False:
-            return domain
-
+    def get(cls, model_name, mode='read'):
+        "Return dictionary of non-global and global rules"
         pool = Pool()
         RuleGroup = pool.get('ir.rule.group')
         Model = pool.get('ir.model')
         RuleGroup_Group = pool.get('ir.rule.group-res.group')
         User_Group = pool.get('res.user-res.group')
-
-        cursor = Transaction().connection.cursor()
         rule_table = cls.__table__()
         rule_group = RuleGroup.__table__()
         rule_group_group = RuleGroup_Group.__table__()
         user_group = User_Group.__table__()
         model = Model.__table__()
-        user_id = Transaction().user
+        transaction = Transaction()
+
+        assert mode in cls.modes
+
+        cursor = transaction.connection.cursor()
+        user_id = transaction.user
+        # root user above constraint
+        if user_id == 0:
+            user_id = transaction.context.get('user')
+            if not user_id:
+                return {}, {}
         cursor.execute(*rule_table.join(rule_group,
                 condition=rule_group.id == rule_table.rule_group
                 ).join(model,
@@ -191,26 +191,7 @@ class Rule(ModelSQL, ModelView):
                     | (rule_group.default_p == True)
                     | (rule_group.global_p == True)
                     )))
-        ids = [x[0] for x in cursor.fetchall()]
-        if not ids:
-            cls._domain_get_cache.set(key, None)
-            return
-        clause = {}
-        clause_global = {}
-        ctx = cls._get_context()
-        # Use root user without context to prevent recursion
-        with Transaction().set_user(0), \
-                Transaction().set_context(user=0):
-            for rule in cls.browse(ids):
-                assert rule.domain, ('Rule domain empty,'
-                    'check if migration was done')
-                dom = PYSONDecoder(ctx).decode(rule.domain)
-                if rule.rule_group.global_p:
-                    clause_global.setdefault(rule.rule_group.id, ['OR'])
-                    clause_global[rule.rule_group.id].append(dom)
-                else:
-                    clause.setdefault(rule.rule_group.id, ['OR'])
-                    clause[rule.rule_group.id].append(dom)
+        ids = [x for x, in cursor]
 
         # Test if there is no rule_group that have no rule
         cursor.execute(*rule_group.join(model,
@@ -222,10 +203,47 @@ class Rule(ModelSQL, ModelView):
                         condition=rule_group_group.group == user_group.group
                         ).select(rule_group_group.rule_group,
                         where=user_group.user == user_id))))
-        fetchone = cursor.fetchone()
-        if fetchone:
-            group_id = fetchone[0]
-            clause[group_id] = []
+        no_rules = cursor.fetchone()
+
+        clause = defaultdict(lambda: ['OR'])
+        clause_global = defaultdict(lambda: ['OR'])
+        decoder = PYSONDecoder(cls._get_context())
+        # Use root user without context to prevent recursion
+        with transaction.set_user(0), transaction.set_context(user=0):
+            for rule in cls.browse(ids):
+                assert rule.domain, ('Rule domain empty,'
+                    'check if migration was done')
+                dom = decoder.decode(rule.domain)
+                if rule.rule_group.global_p:
+                    clause_global[rule.rule_group].append(dom)
+                else:
+                    clause[rule.rule_group].append(dom)
+
+            if no_rules:
+                group_id = no_rules[0]
+                clause[RuleGroup(group_id)] = []
+
+        return clause, clause_global
+
+    @classmethod
+    def domain_get(cls, model_name, mode='read'):
+        transaction = Transaction()
+        # root user above constraint
+        if transaction.user == 0:
+            if not transaction.context.get('user'):
+                return
+            with transaction.set_user(Transaction().context['user']):
+                return cls.domain_get(model_name, mode=mode)
+
+        assert mode in cls.modes
+
+        key = (model_name, mode) + cls._get_cache_key()
+        domain = cls._domain_get_cache.get(key, False)
+        if domain is not False:
+            return domain
+
+        clause, clause_global = cls.get(model_name, mode=mode)
+
         clause = list(clause.values())
         if clause:
             clause.insert(0, 'OR')
