@@ -1,23 +1,36 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from string import Template
 import time
-from itertools import izip
-from sql import Flavor
 
-from ..model import ModelView, ModelSQL, fields
-from ..tools import datetime_strftime
+from sql import Literal, For
+
+from trytond.exceptions import UserError
+from trytond.model.exceptions import ValidationError
+from ..model import ModelView, ModelSQL, DeactivableMixin, fields, Check
 from ..pyson import Eval, And
 from ..transaction import Transaction
 from ..pool import Pool
-from ..config import CONFIG
+from ..i18n import gettext
 from .. import backend
 
 __all__ = [
     'SequenceType', 'Sequence', 'SequenceStrict',
     ]
 
-sql_sequence = CONFIG.options['db_type'] == 'postgresql'
+sql_sequence = backend.get('Database').has_sequence()
+
+
+class AffixError(ValidationError):
+    pass
+
+
+class MissingError(UserError):
+    pass
+
+
+class LastTimestampError(ValidationError):
+    pass
 
 
 class SequenceType(ModelSQL, ModelView):
@@ -28,7 +41,7 @@ class SequenceType(ModelSQL, ModelView):
     code = fields.Char('Sequence Code', required=True)
 
 
-class Sequence(ModelSQL, ModelView):
+class Sequence(DeactivableMixin, ModelSQL, ModelView):
     "Sequence"
     __name__ = 'ir.sequence'
 
@@ -38,7 +51,6 @@ class Sequence(ModelSQL, ModelView):
         states={
             'readonly': Eval('context', {}).contains('code'),
             })
-    active = fields.Boolean('Active')
     prefix = fields.Char('Prefix')
     suffix = fields.Char('Suffix')
     type = fields.Selection([
@@ -85,44 +97,12 @@ class Sequence(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Sequence, cls).__setup__()
+        table = cls.__table__()
         cls._sql_constraints += [
-            ('check_timestamp_rounding', 'CHECK(timestamp_rounding > 0)',
+            ('check_timestamp_rounding',
+                Check(table, table.timestamp_rounding > 0),
                 'Timestamp rounding should be greater than 0'),
             ]
-        cls._error_messages.update({
-                'missing': 'Missing sequence.',
-                'invalid_prefix': ('Invalid prefix "%(prefix)s" on sequence '
-                    '"%(sequence)s".'),
-                'invalid_suffix': ('Invalid suffix "%(suffix)s" on sequence '
-                    '"%(sequence)s".'),
-                'future_last_timestamp': ('Last Timestamp cannot be in the '
-                    'future on sequence "%s".'),
-                })
-
-    @classmethod
-    def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
-        table = TableHandler(cursor, cls, module_name)
-
-        # Migration from 2.0 rename number_next into number_next_internal
-        table.column_rename('number_next', 'number_next_internal')
-
-        super(Sequence, cls).__register__(module_name)
-
-        # Migration from 2.0 create sql_sequence
-        if sql_sequence and not cls._strict:
-            sequences = cls.search([])
-            for sequence in sequences:
-                if sequence.type != 'incremental':
-                    continue
-                if not TableHandler.sequence_exist(cursor,
-                        sequence._sql_sequence_name):
-                    sequence.create_sql_sequence(sequence.number_next_internal)
-
-    @staticmethod
-    def default_active():
-        return True
 
     @staticmethod
     def default_type():
@@ -159,14 +139,11 @@ class Sequence(ModelSQL, ModelView):
     def get_number_next(self, name):
         if self.type != 'incremental':
             return
-        cursor = Transaction().cursor
-        sql_name = self._sql_sequence_name
+
+        transaction = Transaction()
         if sql_sequence and not self._strict:
-            cursor.execute('SELECT '
-                'CASE WHEN NOT is_called THEN last_value '
-                    'ELSE last_value + increment_by '
-                'END FROM "%s"' % sql_name)
-            return cursor.fetchone()[0]
+            return transaction.database.sequence_next_number(
+                transaction.connection, self._sql_sequence_name)
         else:
             return self.number_next_internal
 
@@ -176,10 +153,22 @@ class Sequence(ModelSQL, ModelView):
                 'number_next_internal': value,
                 })
 
+    @classmethod
+    def view_attributes(cls):
+        return [
+            ('//group[@id="incremental"]', 'states', {
+                    'invisible': ~Eval('type').in_(['incremental']),
+                    }),
+            ('//group[@id="timestamp"]', 'states', {
+                    'invisible': ~Eval('type').in_(
+                        ['decimal timestamp', 'hexadecimal timestamp']),
+                    }),
+            ]
+
     @classmethod
     def create(cls, vlist):
         sequences = super(Sequence, cls).create(vlist)
-        for sequence, values in izip(sequences, vlist):
+        for sequence, values in zip(sequences, vlist):
             if sql_sequence and not cls._strict:
                 sequence.update_sql_sequence(values.get('number_next',
                         cls.default_number_next()))
@@ -211,24 +200,22 @@ class Sequence(ModelSQL, ModelView):
     @classmethod
     def validate(cls, sequences):
         super(Sequence, cls).validate(sequences)
-        cls.check_prefix_suffix(sequences)
+        cls.check_affixes(sequences)
         cls.check_last_timestamp(sequences)
 
     @classmethod
-    def check_prefix_suffix(cls, sequences):
+    def check_affixes(cls, sequences):
         "Check prefix and suffix"
-
         for sequence in sequences:
-            for fix, error_message in ((sequence.prefix, 'invalid_prefix'),
-                    (sequence.suffix, 'invalid_suffix')):
+            for affix, error_message in [
+                    (sequence.prefix, 'msg_sequence_invalid_prefix'),
+                    (sequence.suffix, 'msg_sequence_invalid_suffix')]:
                 try:
-                    cls._process(sequence.prefix)
-                    cls._process(sequence.suffix)
-                except Exception:
-                    cls.raise_user_error(error_message, {
-                            'prefix': fix,
-                            'sequence': sequence.rec_name,
-                            })
+                    cls._process(affix)
+                except (TypeError, ValueError) as exc:
+                    raise AffixError(gettext('ir.%s' % error_message,
+                            affix=affix,
+                            sequence=sequence.rec_name)) from exc
 
     @classmethod
     def check_last_timestamp(cls, sequences):
@@ -238,8 +225,8 @@ class Sequence(ModelSQL, ModelView):
             next_timestamp = cls._timestamp(sequence)
             if (sequence.last_timestamp is not None
                     and sequence.last_timestamp > next_timestamp):
-                cls.raise_user_error('future_last_timestamp', (
-                        sequence.rec_name,))
+                raise LastTimestampError(
+                    gettext('ir.msg_sequence_last_timestamp_future'))
 
     @property
     def _sql_sequence_name(self):
@@ -248,22 +235,22 @@ class Sequence(ModelSQL, ModelView):
 
     def create_sql_sequence(self, number_next=None):
         'Create the SQL sequence'
-        cursor = Transaction().cursor
-        param = Flavor.get().param
+        transaction = Transaction()
+
         if self.type != 'incremental':
             return
         if number_next is None:
             number_next = self.number_next
-        cursor.execute('CREATE SEQUENCE "' + self._sql_sequence_name
-            + '" INCREMENT BY ' + param + ' START WITH ' + param,
-            (self.number_increment, number_next))
+        if sql_sequence:
+            transaction.database.sequence_create(transaction.connection,
+                self._sql_sequence_name, self.number_increment, number_next)
 
     def update_sql_sequence(self, number_next=None):
         'Update the SQL sequence'
-        TableHandler = backend.get('TableHandler')
-        cursor = Transaction().cursor
-        param = Flavor.get().param
-        exist = TableHandler.sequence_exist(cursor, self._sql_sequence_name)
+        transaction = Transaction()
+
+        exist = transaction.database.sequence_exist(
+            transaction.connection, self._sql_sequence_name)
         if self.type != 'incremental':
             if exist:
                 self.delete_sql_sequence()
@@ -273,32 +260,37 @@ class Sequence(ModelSQL, ModelView):
             return
         if number_next is None:
             number_next = self.number_next
-        cursor.execute('ALTER SEQUENCE "' + self._sql_sequence_name
-            + '" INCREMENT BY ' + param + ' RESTART WITH ' + param,
-            (self.number_increment, number_next))
+        transaction.database.sequence_update(transaction.connection,
+            self._sql_sequence_name, self.number_increment, number_next)
 
     def delete_sql_sequence(self):
         'Delete the SQL sequence'
-        cursor = Transaction().cursor
+        transaction = Transaction()
         if self.type != 'incremental':
             return
-        cursor.execute('DROP SEQUENCE "%s"'
-            % self._sql_sequence_name)
+        transaction.database.sequence_delete(
+            transaction.connection, self._sql_sequence_name)
 
-    @staticmethod
-    def _process(string, date=None):
+    @classmethod
+    def _process(cls, string, date=None):
+        return Template(string or '').substitute(
+            **cls._get_substitutions(date))
+
+    @classmethod
+    def _get_substitutions(cls, date):
+        '''
+        Returns a dictionary with the keys and values of the substitutions
+        available to format the sequence
+        '''
         pool = Pool()
         Date = pool.get('ir.date')
         if not date:
             date = Date.today()
-        year = datetime_strftime(date, '%Y')
-        month = datetime_strftime(date, '%m')
-        day = datetime_strftime(date, '%d')
-        return Template(string or '').substitute(
-                year=year,
-                month=month,
-                day=day,
-                )
+        return {
+            'year': date.strftime('%Y'),
+            'month': date.strftime('%m'),
+            'day': date.strftime('%d'),
+            }
 
     @staticmethod
     def _timestamp(sequence):
@@ -309,28 +301,26 @@ class Sequence(ModelSQL, ModelView):
     def _get_sequence(cls, sequence):
         if sequence.type == 'incremental':
             if sql_sequence and not cls._strict:
-                cursor = Transaction().cursor
+                cursor = Transaction().connection.cursor()
                 cursor.execute('SELECT nextval(\'"%s"\')'
                     % sequence._sql_sequence_name)
                 number_next, = cursor.fetchone()
             else:
-                #Pre-fetch number_next
+                # Pre-fetch number_next
                 number_next = sequence.number_next_internal
 
-                with Transaction().set_user(0):
-                    cls.write([sequence], {
-                            'number_next_internal': (number_next
-                                + sequence.number_increment),
-                            })
+                cls.write([sequence], {
+                        'number_next_internal': (number_next
+                            + sequence.number_increment),
+                        })
             return '%%0%sd' % sequence.padding % number_next
         elif sequence.type in ('decimal timestamp', 'hexadecimal timestamp'):
             timestamp = sequence.last_timestamp
             while timestamp == sequence.last_timestamp:
                 timestamp = cls._timestamp(sequence)
-            with Transaction().set_user(0):
-                cls.write([sequence], {
-                    'last_timestamp': timestamp,
-                    })
+            cls.write([sequence], {
+                'last_timestamp': timestamp,
+                })
             if sequence.type == 'decimal timestamp':
                 return '%d' % timestamp
             else:
@@ -338,28 +328,41 @@ class Sequence(ModelSQL, ModelView):
         return ''
 
     @classmethod
-    def get_id(cls, domain):
+    def get_id(cls, domain, _lock=False):
         '''
         Return sequence value for the domain
         '''
         if isinstance(domain, cls):
             domain = domain.id
-        if isinstance(domain, (int, long)):
+        if isinstance(domain, int):
             domain = [('id', '=', domain)]
 
         # bypass rules on sequences
-        with Transaction().set_context(user=False):
+        with Transaction().set_context(user=False, _check_access=False):
             with Transaction().set_user(0):
                 try:
                     sequence, = cls.search(domain, limit=1)
                 except TypeError:
-                    cls.raise_user_error('missing')
-            date = Transaction().context.get('date')
-            return '%s%s%s' % (
-                cls._process(sequence.prefix, date=date),
-                cls._get_sequence(sequence),
-                cls._process(sequence.suffix, date=date),
-                )
+                    raise MissingError(gettext('ir.msg_sequence_missing'))
+                if _lock:
+                    transaction = Transaction()
+                    database = transaction.database
+                    connection = transaction.connection
+                    if not database.has_select_for():
+                        database.lock(connection, cls._table)
+                    else:
+                        table = cls.__table__()
+                        query = table.select(Literal(1),
+                            where=table.id == sequence.id,
+                            for_=For('UPDATE', nowait=True))
+                        cursor = connection.cursor()
+                        cursor.execute(*query)
+                date = Transaction().context.get('date')
+                return '%s%s%s' % (
+                    cls._process(sequence.prefix, date=date),
+                    cls._get_sequence(sequence),
+                    cls._process(sequence.suffix, date=date),
+                    )
 
     @classmethod
     def get(cls, code):
@@ -369,10 +372,9 @@ class Sequence(ModelSQL, ModelView):
 class SequenceStrict(Sequence):
     "Sequence Strict"
     __name__ = 'ir.sequence.strict'
-    _table = 'ir_sequence_strict'  # Needed to override Sequence._table
+    _table = None  # Needed to reset Sequence._table
     _strict = True
 
     @classmethod
     def get_id(cls, clause):
-        Transaction().cursor.lock(cls._table)
-        return super(SequenceStrict, cls).get_id(clause)
+        return super(SequenceStrict, cls).get_id(clause, _lock=True)
