@@ -538,11 +538,13 @@ class ModelSQL(ModelStorage):
                         '%s,%s' % (cls.__name__, id_))
                 except KeyError:
                     continue
-                sql_type = fields.Numeric('timestamp').sql_type().base
+                if timestamp is None:
+                    continue
+                sql_type = fields.Char('timestamp').sql_type().base
                 where.append((table.id == id_)
                     & (Extract('EPOCH',
                             Coalesce(table.write_date, table.create_date)
-                            ).cast(sql_type) > timestamp))
+                            ).cast(sql_type) != timestamp))
             if where:
                 cursor.execute(*table.select(table.id, where=where, limit=1))
                 if cursor.fetchone():
@@ -1588,6 +1590,8 @@ class ModelSQL(ModelStorage):
                                 clause &= Literal(False)
                             clause &= operator(column, value)
                         where |= clause
+                    if isinstance(sql, Exclude) and sql.where:
+                        where &= sql.where
                     cursor.execute(
                         *table.select(table.id, where=where, limit=1))
                     if cursor.fetchone():
