@@ -1,14 +1,17 @@
 # -*- coding: utf-8 -*-
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import http.client
 import logging
 import pydoc
+import time
 try:
     from http import HTTPStatus
 except ImportError:
     from http import client as HTTPStatus
 
 from werkzeug.exceptions import abort
+from werkzeug.wrappers import Response
 from sql import Table
 
 from trytond import security
@@ -86,6 +89,11 @@ def root(request, *args):
     return methods[request.rpc_method](request, *request.rpc_params)
 
 
+@app.route('/<path:path>', methods=['OPTIONS'])
+def options(request, path):
+    return Response(status=HTTPStatus.NO_CONTENT)
+
+
 def db_exist(request, database_name):
     Database = backend.get('Database')
     try:
@@ -101,7 +109,7 @@ def db_list(request, *args):
     context = {'_request': request.context}
     hostname = get_hostname(request.host)
     with Transaction().start(
-            None, 0, context=context, close=True, _nocache=True
+            None, 0, context=context, readonly=True, close=True,
             ) as transaction:
         return transaction.database.list(hostname=hostname)
 
@@ -152,7 +160,7 @@ def _dispatch(request, pool, *args, **kwargs):
         context = {'_request': request.context}
         if not security.check_timeout(
                 pool.database_name, user, session, context=context):
-            abort(HTTPStatus.UNAUTHORIZED)
+            abort(http.client.UNAUTHORIZED)
 
     log_message = '%s.%s(*%s, **%s) from %s@%s/%s'
     username = request.authorization.username
@@ -162,7 +170,10 @@ def _dispatch(request, pool, *args, **kwargs):
         obj, method, args, kwargs, username, request.remote_addr, request.path)
     logger.info(log_message, *log_args)
 
-    for count in range(config.getint('database', 'retry'), -1, -1):
+    retry = config.getint('database', 'retry')
+    for count in range(retry, -1, -1):
+        if count != retry:
+            time.sleep(0.02 * (retry - count))
         with Transaction().start(pool.database_name, user,
                 readonly=rpc.readonly) as transaction:
             try:
@@ -203,4 +214,7 @@ def _dispatch(request, pool, *args, **kwargs):
             context = {'_request': request.context}
             security.reset(pool.database_name, session, context=context)
         logger.debug('Result: %s', result)
-        return result
+        response = app.make_response(request, result)
+        if rpc.readonly and rpc.cache:
+            response.headers.extend(rpc.cache.headers())
+        return response
