@@ -489,10 +489,7 @@ class In(PYSON):
 
     @staticmethod
     def eval(dct, context):
-        if dct['v']:
-            return dct['k'] in dct['v']
-        else:
-            return False
+        return dct['k'] in dct['v']
 
 
 class Date(PYSON):
@@ -613,7 +610,7 @@ class DateTime(Date):
 
     @staticmethod
     def eval(dct, context):
-        return datetime.datetime.utcnow() + relativedelta(
+        return datetime.datetime.now() + relativedelta(
             year=dct['y'],
             month=dct['M'],
             day=dct['d'],
