@@ -1,19 +1,19 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 import encodings.idna
-import urllib
 import socket
+import urllib.parse
 
-from trytond.config import CONFIG
+from trytond.config import config
 from trytond.transaction import Transaction
 
-__all__ = ['URLMixin']
+__all__ = ['URLMixin', 'HOSTNAME']
 
-HOSTNAME = (CONFIG['hostname_jsonrpc']
-    or unicode(socket.getfqdn(), 'utf8'))
-HOSTNAME = '.'.join(encodings.idna.ToASCII(part) if part else ''
-    for part in HOSTNAME.split('.'))
+HOSTNAME = (config.get('web', 'hostname')
+    or socket.getfqdn())
+HOSTNAME = '.'.join(encodings.idna.ToASCII(part).decode('ascii')
+    if part else '' for part in HOSTNAME.split('.'))
 
 
 class URLAccessor(object):
@@ -34,9 +34,9 @@ class URLAccessor(object):
             raise NotImplementedError
 
         url_part['name'] = cls.__name__
-        url_part['database'] = Transaction().cursor.database_name
+        url_part['database'] = Transaction().database.name
 
-        local_part = urllib.quote('%(database)s/%(type)s/%(name)s' % url_part)
+        local_part = urllib.parse.quote('%(database)s/%(type)s/%(name)s' % url_part)
         if isinstance(inst, Model) and inst.id:
             local_part += '/%d' % inst.id
         return 'tryton://%s/%s' % (HOSTNAME, local_part)
