@@ -1,6 +1,7 @@
 CREATE TABLE ir_configuration (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
-    language VARCHAR
+    language VARCHAR,
+    hostname VARCHAR
 );
 
 CREATE TABLE ir_model (
@@ -43,7 +44,6 @@ CREATE TABLE ir_translation (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
     lang VARCHAR,
     src TEXT,
-    src_md5 VARCHAR(32) NOT NULL,
     name VARCHAR,
     res_id INTEGER,
     value TEXT,
@@ -57,6 +57,7 @@ CREATE TABLE ir_lang (
     name VARCHAR,
     code VARCHAR,
     translatable BOOLEAN,
+    parent VARCHAR,
     active BOOLEAN,
     direction VARCHAR
 );
@@ -82,7 +83,7 @@ CREATE TABLE "res_user-res_group" (
     "group" INTEGER
 );
 
-CREATE TABLE ir_module_module (
+CREATE TABLE ir_module (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
     create_uid INTEGER,
     create_date TIMESTAMP,
@@ -92,7 +93,7 @@ CREATE TABLE ir_module_module (
     state VARCHAR
 );
 
-CREATE TABLE ir_module_module_dependency (
+CREATE TABLE ir_module_dependency (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
     create_uid INTEGER,
     create_date TIMESTAMP,
@@ -101,3 +102,9 @@ CREATE TABLE ir_module_module_dependency (
     name VARCHAR,
     module INTEGER
 );
+
+CREATE TABLE ir_cache (
+    id INTEGER PRIMARY KEY AUTOINCREMENT,
+    name VARCHAR,
+    "timestamp" TIMESTAMP
+);
