@@ -330,6 +330,11 @@ class Fs2bdAccessor:
             self.fetch_new_module(module)
         return self.fs2db[module].get(fs_id, None)
 
+    def exists(self, module, fs_id):
+        if module not in self.fetched_modules:
+            self.fetch_new_module(module)
+        return fs_id in self.fs2db[module]
+
     def get_browserecord(self, module, model_name, db_id):
         if module not in self.fetched_modules:
             self.fetch_new_module(module)
@@ -583,7 +588,7 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
 
         Model = self.pool.get(model)
 
-        if self.fs2db.get(module, fs_id):
+        if self.fs2db.exists(module, fs_id):
 
             # Remove this record from the to_delete list. This means that
             # the corresponding record have been found.
@@ -599,6 +604,10 @@ class TrytondXmlHandler(sax.handler.ContentHandler):
                 self.fs2db.get(module, fs_id)[x]
                 for x in ["db_id", "model", "id", "values"]]
 
+            # Check if record has not been deleted
+            if db_id is None:
+                return
+
             if not old_values:
                 old_values = {}
             else:
@@ -803,9 +812,11 @@ def post_import(pool, module, to_delete):
         except Exception:
             transaction.rollback()
             logger.error(
-                "Could not delete id %d from model %s.\n"
-                "There may be a relation that points to this resource "
-                "that must be manually fixed before restarting the update.",
+                'Could not delete id: %d of model %s\n'
+                'There should be some relation '
+                'that points to this resource\n'
+                'You should manually fix this '
+                'and restart --update=module\n',
                 db_id, model, exc_info=True)
             if 'active' in Model._fields:
                 try:
