@@ -282,6 +282,7 @@ class ModelSQLTestCase(unittest.TestCase):
             # timestamp precision of sqlite is the second
             time.sleep(1)
 
+        transaction.timestamp[str(record)] = timestamp
         ModelsqlTimestamp.write([record], {})
         transaction.commit()
 
@@ -293,6 +294,10 @@ class ModelSQLTestCase(unittest.TestCase):
         self.assertRaises(ConcurrencyException,
             ModelsqlTimestamp.delete, [record])
 
+        transaction.timestamp[str(record)] = None
+        ModelsqlTimestamp.write([record], {})
+        transaction.commit()
+
         transaction.timestamp.pop(str(record), None)
         ModelsqlTimestamp.write([record], {})
         transaction.commit()
@@ -462,7 +467,20 @@ class ModelSQLTestCase(unittest.TestCase):
         pool = Pool()
         Model = pool.get('test.modelsql.exclude')
 
-        records = Model.create([{'value': -1}, {'value': -1}])
+        records = Model.create([{'value': 1, 'condition': False}] * 2)
+
+        self.assertEqual(len(records), 2)
+
+    @with_transaction()
+    def test_constraint_exclude_exclusion_mixed(self):
+        "Test exclude constraint exclusion mixed"
+        pool = Pool()
+        Model = pool.get('test.modelsql.exclude')
+
+        records = Model.create([
+                {'value': 1, 'condition': False},
+                {'value': 1, 'condition': True},
+                ])
 
         self.assertEqual(len(records), 2)
 
